"""
Enhanced features integration for Resume ATS Plugin.
"""

import os
import json
from typing import Dict, List, Any, Optional, Tuple
from io import BytesIO
import logging
import tempfile

from .core import ResumeATS, analyze_resume, optimize_resume
# We're using the placeholder here since we've implemented the real logic in this file
from .pdf_reporter import generate_pdf_report
from .resume_anonymizer import ResumeAnonymizer, anonymize_resume_text, anonymize_resume_file
from .resume_version_tracker import ResumeVersionTracker
from .resume_comparer import ResumeComparer, compare_resumes, compare_resume_to_jobs

# Initialize logging
logger = logging.getLogger(__name__)


class EnhancedResumeATS(ResumeATS):
    """
    Extended ResumeATS class with enhanced features including PDF reports,
    resume anonymization, version tracking, and comparison tools.
    """
    
    def __init__(self, config: Optional[Dict] = None):
        """
        Initialize the enhanced ResumeATS.
        
        Args:
            config: Optional configuration dictionary
        """
        super().__init__(config)
        
        # Initialize component classes
        self.anonymizer = ResumeAnonymizer(config)
        self.version_tracker = ResumeVersionTracker(
            config.get("version_storage_dir") if config else None
        )
        self.comparer = ResumeComparer(config)
    
    def analyze_and_report(self, resume_path: str, job_description: Optional[str] = None, 
                          output_pdf_path: Optional[str] = None) -> Tuple[Dict[str, Any], str]:
        """
        Analyze a resume and generate a PDF report.
        
        Args:
            resume_path: Path to the resume file
            job_description: Optional job description text
            output_pdf_path: Optional path for the PDF report (default: auto-generated)
            
        Returns:
            Tuple of (analysis results, PDF report path)
        """
        # Analyze the resume
        analysis = self.analyze(resume_path, job_description)
        
        # Generate default PDF path if not provided
        if output_pdf_path is None:
            base_name = os.path.splitext(resume_path)[0]
            output_pdf_path = f"{base_name}_ats_report.pdf"
        
        # Generate PDF report
        pdf_path = generate_pdf_report(analysis, output_pdf_path)
        
        # Track this version if job description is provided (more meaningful comparison)
        if job_description and self.config.get("auto_track_versions", True):
            try:
                if os.path.exists(resume_path):  # Only try to track if file exists
                    version_id = self.version_tracker.save_version(resume_path, analysis)
                    analysis["version_id"] = version_id
                    logger.info(f"Resume version tracked with ID: {version_id}")
            except Exception as e:
                logger.warning(f"Failed to track resume version: {e}")
        
        return analysis, pdf_path
    
    def optimize_and_report(self, resume_path: str, job_description: str,
                           output_pdf_path: Optional[str] = None) -> Tuple[Dict[str, Any], str]:
        """
        Optimize a resume and generate a PDF report.
        
        Args:
            resume_path: Path to the resume file
            job_description: Job description text
            output_pdf_path: Optional path for the PDF report
            
        Returns:
            Tuple of (optimization results, PDF report path)
        """
        # Get optimization suggestions
        optimization = self.optimize(resume_path, job_description)
        
        # Generate default PDF path if not provided
        if output_pdf_path is None:
            base_name = os.path.splitext(resume_path)[0]
            output_pdf_path = f"{base_name}_optimization_report.pdf"
        
        # Generate PDF report
        pdf_path = generate_pdf_report(optimization, output_pdf_path)
        
        return optimization, pdf_path
    
    def anonymize_resume(self, resume_path: str, output_path: Optional[str] = None,
                        anonymization_config: Optional[Dict] = None) -> Tuple[str, Dict[str, List[str]]]:
        """
        Anonymize a resume by removing personally identifiable information.
        
        Args:
            resume_path: Path to the resume file
            output_path: Optional path for the anonymized resume
            anonymization_config: Optional specific anonymization configuration
            
        Returns:
            Tuple of (path to anonymized resume, dict of replaced items)
        """
        # Create anonymizer with custom config if provided
        if anonymization_config:
            anonymizer = ResumeAnonymizer(anonymization_config)
        else:
            anonymizer = self.anonymizer
        
        # Anonymize the resume
        return anonymizer.anonymize_file(resume_path, output_path)
    
    def track_version(self, resume_path: str, analysis_result: Dict[str, Any],
                     version_name: Optional[str] = None) -> str:
        """
        Track a resume version.
        
        Args:
            resume_path: Path to the resume file
            analysis_result: Analysis result from analyze()
            version_name: Optional name for this version
            
        Returns:
            Version ID
        """
        return self.version_tracker.save_version(resume_path, analysis_result, version_name)
    
    def get_version_history(self) -> List[Dict[str, Any]]:
        """
        Get all tracked resume versions.
        
        Returns:
            List of version data
        """
        return self.version_tracker.get_versions()
    
    def compare_versions(self, version_id1: str, version_id2: str) -> Dict[str, Any]:
        """
        Compare two tracked resume versions.
        
        Args:
            version_id1: First version ID
            version_id2: Second version ID
            
        Returns:
            Comparison data
        """
        return self.version_tracker.compare_versions(version_id1, version_id2)
    
    def compare_multiple_resumes(self, resume_paths: List[str], 
                                job_description: Optional[str] = None) -> Dict[str, Any]:
        """
        Compare multiple resumes, optionally against a job description.
        
        Args:
            resume_paths: List of paths to resume files
            job_description: Optional job description
            
        Returns:
            Comparison results
        """
        return self.comparer.compare_multiple_resumes(resume_paths, job_description)
    
    def compare_resume_to_multiple_jobs(self, resume_path: str, 
                                       job_descriptions: Dict[str, str]) -> Dict[str, Any]:
        """
        Compare a single resume against multiple job descriptions.
        
        Args:
            resume_path: Path to the resume file
            job_descriptions: Dictionary mapping job names to descriptions
            
        Returns:
            Comparison results
        """
        return self.comparer.compare_resume_to_multiple_jobs(resume_path, job_descriptions)
    
    def generate_comparison_charts(self, comparison_result: Dict[str, Any], 
                                 output_dir: Optional[str] = None) -> Dict[str, str]:
        """
        Generate visualization charts for resume comparison and save to files.
        
        Args:
            comparison_result: Comparison data from compare_multiple_resumes
            output_dir: Optional directory to save charts (default: current directory)
            
        Returns:
            Dictionary mapping chart names to file paths
        """
        # Generate the charts
        chart_buffers = self.comparer.generate_comparison_charts(comparison_result)
        
        # Set output directory
        if output_dir is None:
            output_dir = os.getcwd()
        os.makedirs(output_dir, exist_ok=True)
        
        # Save each chart to a file
        chart_paths = {}
        for chart_name, buffer in chart_buffers.items():
            file_path = os.path.join(output_dir, f"{chart_name}.png")
            
            with open(file_path, 'wb') as f:
                f.write(buffer.getvalue())
            
            chart_paths[chart_name] = file_path
        
        return chart_paths


# Enhanced convenience functions
def analyze_and_report(resume_path: str, job_description: Optional[str] = None,
                      output_pdf_path: Optional[str] = None,
                      config: Optional[Dict] = None) -> Tuple[Dict[str, Any], str]:
    """
    Analyze a resume and generate a PDF report.
    
    Args:
        resume_path: Path to the resume file
        job_description: Optional job description
        output_pdf_path: Optional path for the PDF report
        config: Optional configuration
        
    Returns:
        Tuple of (analysis results, PDF report path)
    """
    # Run the analyze function first
    analysis = analyze_resume(resume_path, job_description, config)
    
    # Generate the PDF report
    if output_pdf_path is None:
        base_name = os.path.splitext(resume_path)[0]
        output_pdf_path = f"{base_name}_ats_report.pdf"
    
    pdf_path = generate_pdf_report(analysis, output_pdf_path)
    
    return analysis, pdf_path


def optimize_and_report(resume_path: str, job_description: str,
                       output_pdf_path: Optional[str] = None,
                       config: Optional[Dict] = None) -> Tuple[Dict[str, Any], str]:
    """
    Optimize a resume and generate a PDF report.
    
    Args:
        resume_path: Path to the resume file
        job_description: Job description
        output_pdf_path: Optional path for the PDF report
        config: Optional configuration
        
    Returns:
        Tuple of (optimization results, PDF report path)
    """
    # Run the optimize function
    optimization = optimize_resume(resume_path, job_description, config)
    
    # Generate the PDF report
    if output_pdf_path is None:
        base_name = os.path.splitext(resume_path)[0]
        output_pdf_path = f"{base_name}_optimization_report.pdf"
    
    pdf_path = generate_pdf_report(optimization, output_pdf_path)
    
    return optimization, pdf_path